@@ -318,7 +318,7 @@ Model <- R6::R6Class("Model",
                          if(is.null(weights)){
                            self$weights <- rep(1,nrow(self$mean$data))
                          } else {
-                           self$weights <- offset
+                           self$weights <- weights
                          }
                          if(self$family[[1]]=="binomial"){
                            if(is.null(trials) || all(trials == 1)){
@@ -718,12 +718,16 @@ Model <- R6::R6Class("Model",
                        #'between iterations at which to stop the algorithm.
                        #'@param max.iter Integer. The maximum number of iterations of the MCML algorithm.
                        #'@param se String. Type of standard error to return. Options are "gls" for GLS standard errors (the default),
-                       #' "robust" for Huber robust sandwich estimator, and "kr" for Kenward-Roger bias corrected standard errors.
+                       #' "robust" for Huber robust sandwich estimator, "kr" for Kenward-Roger bias corrected standard errors, "bw" to use
+                       #' GLS standard errors with a between-within correction to the degrees of freedom, "bwrobust" to use robust 
+                       #' standard errors with between-within correction to the degrees of freedom.
                        #'@param sparse Logical indicating whether to use sparse matrix methods
                        #'@param usestan Logical whether to use Stan (through the package `cmdstanr`) for the MCMC sampling. If FALSE then
                        #'the internal Hamiltonian Monte Carlo sampler will be used instead. We recommend Stan over the internal sampler as
                        #'it generally produces a larger number of effective samplers per unit time, especially for more complex
                        #'covariance functions.
+                       #'@param se.theta Logical. Whether to calculate the standard errors for the covariance parameters. This step is a slow part
+                       #' of the calculation, so can be disabled if required in larger models. Has no effect for Kenward-Roger standard errors.
                        #'@return A `mcml` object
                        #'@seealso \link[glmmrBase]{Model}, \link[glmmrBase]{Covariance}, \link[glmmrBase]{MeanFunction}
                        #'@examples
@@ -763,7 +767,8 @@ Model <- R6::R6Class("Model",
                                        max.iter = 30,
                                        se = "gls",
                                        sparse = FALSE,
-                                       usestan = TRUE){
+                                       usestan = TRUE,
+                                       se.theta = TRUE){
                          private$verify_data(y)
                          private$set_y(y)
                          Model__use_attenuation(private$ptr,private$attenuate_parameters)
@@ -778,6 +783,7 @@ Model <- R6::R6Class("Model",
                          theta <- self$covariance$parameters
                          var_par <- self$var_par
                          var_par_family <- I(self$family[[1]]%in%c("gaussian","Gamma","beta"))
+                         ncovpar <- ifelse(var_par_family,length(theta)+1,length(theta))
                          all_pars <- c(beta,theta)
                          if(var_par_family)all_pars <- c(all_pars,var_par)
                          all_pars_new <- rep(1,length(all_pars))
@@ -833,7 +839,6 @@ Model <- R6::R6Class("Model",
                                   file=tempfile())
                              dsamps <- fit$draws("gamma",format = "matrix")
                              class(dsamps) <- "matrix"
-                             #dsamps <- Matrix::Matrix(L %*% Matrix::t(dsamps)) #check this
                              Model__update_u(private$ptr,as.matrix(t(dsamps)))
                            } else {
                              Model__mcmc_sample(private$ptr,
@@ -859,7 +864,6 @@ Model <- R6::R6Class("Model",
                            if(trace==2)t3 <- Sys.time()
                            if(trace==2)cat("\nModel fitting took: ",t3-t2,"s")
                            if(verbose){
-                             #cat("\ntheta:",theta[all_pars])
                              cat("\nBeta: ", beta_new)
                              cat("\nTheta: ", theta_new)
                              if(var_par_family)cat("\nSigma: ",var_par_new)
@@ -882,22 +886,29 @@ Model <- R6::R6Class("Model",
                          if(verbose)cat("\n\nCalculating standard errors...\n")
                          self$var_par <- var_par_new
                          u <- Model__u(private$ptr, TRUE)
-                         if(se == "gls"){
+                         if(se == "gls" || se == "bw"){
                            M <- Matrix::solve(Model__obs_information_matrix(private$ptr))[1:length(beta),1:length(beta)]
-                           SE_theta <- sqrt(diag(solve(Model__infomat_theta(private$ptr))))
-                         } else if(se == "robust"){
+                           if(se.theta){
+                             SE_theta <- tryCatch(sqrt(diag(solve(Model__infomat_theta(private$ptr)))), error = rep(NA, ncovpar))
+                           } else {
+                             SE_theta <- rep(NA, ncovpar)
+                           }
+                         } else if(se == "robust" || se == "bwrobust"){
                            M <- Model__sandwich(private$ptr)
-                           SE_theta <- sqrt(diag(solve(Model__infomat_theta(private$ptr))))
+                           if(se.theta){
+                             SE_theta <- tryCatch(sqrt(diag(solve(Model__infomat_theta(private$ptr)))), error = rep(NA, ncovpar))
+                           } else {
+                             SE_theta <- rep(NA, ncovpar)
+                           }
                          } else if(se == "kr"){
                            Mout <- Model__kenward_roger(private$ptr)
                            M <- Mout[[1]]
                            SE_theta <- sqrt(diag(Mout[[2]]))
                          }
-                         SE <- sqrt(Matrix::diag(M))
-                         SE_theta <- sqrt(diag(solve(Model__infomat_theta(private$ptr))))
+                         SE <- sqrt(diag(M))
                          repar_table <- self$covariance$parameter_table()
                          beta_names <- Model__beta_parameter_names(private$ptr)
-                         theta_names <- repar_table$term#unique(Model__theta_parameter_names(private$ptr))
+                         theta_names <- repar_table$term
                          if(self$family[[1]]%in%c("Gamma","beta")){
                            mf_pars_names <- c(beta_names,theta_names,"sigma")
                            SE <- c(SE,rep(NA,length(theta_new)+1))
@@ -908,9 +919,35 @@ Model <- R6::R6Class("Model",
                          }
                          res <- data.frame(par = c(mf_pars_names,paste0("d",1:nrow(u))),
                                            est = c(all_pars_new,rowMeans(u)),
-                                           SE=c(SE,rep(NA,nrow(u))))
-                         res$lower <- res$est - qnorm(1-0.05/2)*res$SE
-                         res$upper <- res$est + qnorm(1-0.05/2)*res$SE
+                                           SE=c(SE,rep(NA,nrow(u))),
+                                           t = NA,
+                                           p = NA,
+                                           lower = NA,
+                                           upper = NA)
+                         dof <- rep(self$n(),length(beta))
+                         if(se == "kr"){
+                           for(i in 1:length(beta)){
+                             if(!is.na(res$SE[i])){
+                               res$t[i] <- (res$est[i]/res$SE[i])#*sqrt(lambda)
+                               res$p[i] <- 2*(1-stats::pt(abs(res$t[i]),Mout$dof[i],lower.tail=FALSE))
+                               res$lower[i] <- res$est[i] - stats::qt(0.975,Mout$dof[i],lower.tail=FALSE)*res$SE[i]
+                               res$upper[i] <- res$est[i] + stats::qt(0.975,Mout$dof[i],lower.tail=FALSE)*res$SE[i]
+                             }
+                             dof[i] <- Mout$dof[i]
+                           }
+                         } else if(se=="bw" || se == "bwrobust"){
+                           res$t <- res$est/res$SE
+                           bwdof <- sum(repar_table$count) - length(beta)
+                           res$p <- 2*(1-stats::pt(abs(res$t),bwdof,lower.tail=FALSE))
+                           res$lower <- res$est - qt(1-0.05/2,bwdof,lower.tail=FALSE)*res$SE
+                           res$upper <- res$est + qt(1-0.05/2,bwdof,lower.tail=FALSE)*res$SE
+                           dof <- rep(bwdof,length(beta))
+                         } else {
+                           res$t <- res$est/res$SE
+                           res$p <- 2*(1-stats::pnorm(abs(res$t)))
+                           res$lower <- res$est - qnorm(1-0.05/2)*res$SE
+                           res$upper <- res$est + qnorm(1-0.05/2)*res$SE
+                         }
                          repar_table <- repar_table[!duplicated(repar_table$id),]
                          rownames(u) <- rep(repar_table$term,repar_table$count)
                          aic <- Model__aic(private$ptr)
@@ -936,9 +973,11 @@ Model <- R6::R6Class("Model",
                                      link = self$family[[2]],
                                      re.samps = u,
                                      iter = iter,
+                                     dof = dof,
                                      P = length(self$mean$parameters),
                                      Q = length(self$covariance$parameters),
-                                     var_par_family = var_par_family)
+                                     var_par_family = var_par_family,
+                                     y=y)
                          class(out) <- "mcml"
                          return(out)
                        },
@@ -959,9 +998,13 @@ Model <- R6::R6Class("Model",
                        #'@param method String. Either "nloptim" for non-linear optimisation, or "nr" for Newton-Raphson (default) algorithm
                        #'@param verbose logical indicating whether to provide detailed algorithm feedback (default is TRUE).
                        #'@param se String. Type of standard error to return. Options are "gls" for GLS standard errors (the default),
-                       #' "robust" for Huber robust sandwich estimator, and "kr" for Kenward-Roger bias corrected standard errors.
+                       #' "robust" for Huber robust sandwich estimator, "kr" for Kenward-Roger bias corrected standard errors, "bw" to use
+                       #' GLS standard errors with a between-within correction to the degrees of freedom, "bwrobust" to use robust 
+                       #' standard errors with between-within correction to the degrees of freedom.
                        #'@param max.iter Maximum number of algorithm iterations, default 20.
                        #'@param tol Maximum difference between successive iterations at which to terminate the algorithm
+                       #'@param se.theta Logical. Whether to calculate the standard errors for the covariance parameters. This step is a slow part
+                       #' of the calculation, so can be disabled if required in larger models. Has no effect for Kenward-Roger standard errors.
                        #'@return A `mcml` object
                        #' @seealso \link[glmmrBase]{Model}, \link[glmmrBase]{Covariance}, \link[glmmrBase]{MeanFunction}
                        #'@examples
@@ -990,7 +1033,8 @@ Model <- R6::R6Class("Model",
                                      verbose = FALSE,
                                      se = "gls",
                                      max.iter = 40,
-                                     tol = 1e-2){
+                                     tol = 1e-4,
+                                     se.theta = TRUE){
                          private$verify_data(y)
                          private$set_y(y)
                          Model__use_attenuation(private$ptr,private$attenuate_parameters)
@@ -1001,6 +1045,7 @@ Model <- R6::R6Class("Model",
                          trace <- ifelse(verbose,2,0)
                          beta <- self$mean$parameters
                          theta <- self$covariance$parameters
+                         ncovpar <- ifelse(var_par_family,length(theta)+1,length(theta))
                          var_par <- self$var_par
                          all_pars <- c(beta,theta)
                          if(var_par_family)all_pars <- c(all_pars,var_par)
@@ -1042,12 +1087,20 @@ Model <- R6::R6Class("Model",
                          self$var_par <- var_par_new
                          u <- Model__u(private$ptr,TRUE)
                          if(verbose)cat("\n\nCalculating standard errors...\n")
-                         if(se == "gls"){
+                         if(se == "gls" || se =="bw"){
                            M <- Matrix::solve(Model__obs_information_matrix(private$ptr))[1:length(beta),1:length(beta)]
-                           SE_theta <- sqrt(diag(solve(Model__infomat_theta(private$ptr))))
-                         } else if(se == "robust"){
+                           if(se.theta){
+                             SE_theta <- tryCatch(sqrt(diag(solve(Model__infomat_theta(private$ptr)))), error = rep(NA, ncovpar))
+                           } else {
+                             SE_theta <- rep(NA, ncovpar)
+                           }
+                         } else if(se == "robust" || se == "bwrobust"){
                            M <- Model__sandwich(private$ptr)
-                           SE_theta <- sqrt(diag(solve(Model__infomat_theta(private$ptr))))
+                           if(se.theta){
+                             SE_theta <- tryCatch(sqrt(diag(solve(Model__infomat_theta(private$ptr)))), error = rep(NA, ncovpar))
+                           } else {
+                             SE_theta <- rep(NA, ncovpar)
+                           }
                          } else if(se == "kr"){
                            Mout <- Model__kenward_roger(private$ptr)
                            M <- Mout[[1]]
@@ -1066,10 +1119,37 @@ Model <- R6::R6Class("Model",
                            SE <- c(SE,SE_theta)
                          }
                          res <- data.frame(par = c(mf_pars_names,paste0("d",1:nrow(u))),
-                                           est = c(all_pars_new,drop(u)),
-                                           SE=c(SE,rep(NA,nrow(u))))
-                         res$lower <- res$est - qnorm(1-0.05/2)*res$SE
-                         res$upper <- res$est + qnorm(1-0.05/2)*res$SE
+                                           est = c(all_pars_new,rowMeans(u)),
+                                           SE=c(SE,rep(NA,nrow(u))),
+                                           t = NA,
+                                           p = NA,
+                                           lower = NA,
+                                           upper = NA)
+                         
+                         dof <- rep(self$n(),length(beta))
+                         if(se == "kr"){
+                           for(i in 1:length(beta)){
+                             if(!is.na(res$SE[i])){
+                               res$t[i] <- (res$est[i]/res$SE[i])#*sqrt(lambda)
+                               res$p[i] <- 2*(1-stats::pt(abs(res$t[i]),Mout$dof[i],lower.tail=FALSE))
+                               res$lower[i] <- res$est[i] - stats::qt(0.975,Mout$dof[i],lower.tail=FALSE)*res$SE[i]
+                               res$upper[i] <- res$est[i] + stats::qt(0.975,Mout$dof[i],lower.tail=FALSE)*res$SE[i]
+                             }
+                             dof[i] <- Mout$dof[i]
+                           }
+                         } else if(se=="bw" || se == "bwrobust"){
+                           res$t <- res$est/res$SE
+                           bwdof <- sum(repar_table$count) - length(beta)
+                           res$p <- 2*(1-stats::pt(abs(res$t),bwdof,lower.tail=FALSE))
+                           res$lower <- res$est - qt(1-0.05/2,bwdof,lower.tail=FALSE)*res$SE
+                           res$upper <- res$est + qt(1-0.05/2,bwdof,lower.tail=FALSE)*res$SE
+                           dof <- rep(bwdof,length(beta))
+                         } else {
+                           res$t <- res$est/res$SE
+                           res$p <- 2*(1-stats::pnorm(abs(res$t)))
+                           res$lower <- res$est - qnorm(1-0.05/2)*res$SE
+                           res$upper <- res$est + qnorm(1-0.05/2)*res$SE
+                         }
                          repar_table <- repar_table[!duplicated(repar_table$id),]
                          rownames(u) <- rep(repar_table$term,repar_table$count)
                          aic <- Model__aic(private$ptr)
@@ -1095,9 +1175,11 @@ Model <- R6::R6Class("Model",
                                      link = self$family[[2]],
                                      re.samps = u,
                                      iter = iter,
+                                     dof = dof,
                                      P = length(self$mean$parameters),
                                      Q = length(self$covariance$parameters),
-                                     var_par_family = var_par_family)
+                                     var_par_family = var_par_family,
+                                     y = y)
                          class(out) <- "mcml"
                          return(out)
                        },
@@ -1273,7 +1355,6 @@ Model <- R6::R6Class("Model",
                                           self$mean$.__enclos_env__$private$hash))
                        },
                        ptr = NULL,
-                       bitsptr = NULL,
                        set_y = function(y){
                          if(is.null(private$ptr))private$update_ptr()
                          Model__set_y(private$ptr,y)
@@ -1292,16 +1373,16 @@ Model <- R6::R6Class("Model",
                              data <- cbind(data,self$mean$data[,cnames])
                            }
                            if(self$family[[1]]=="bernoulli" & any(self$trials>1))self$family[[1]] <- "binomial"
-                           private$bitsptr <- ModelBits__new(form,as.matrix(data),colnames(data),
-                                                             tolower(self$family[[1]]),self$family[[2]],
-                                                             self$mean$parameters,self$covariance$parameters)
-                           private$ptr <- Model__new_from_bits(private$bitsptr)
+                           private$ptr <- Model__new_w_pars(form,as.matrix(data),colnames(data),
+                                                               tolower(self$family[[1]]),self$family[[2]],
+                                                               self$mean$parameters,self$covariance$parameters)
                            Model__set_offset(private$ptr,self$mean$offset)
                            Model__set_weights(private$ptr,self$weights)
                            Model__set_var_par(private$ptr,self$var_par)
                            if(self$family[[1]] == "binomial")Model__set_trials(private$ptr,self$trials)
                            Model__update_beta(private$ptr,self$mean$parameters)
                            Model__update_theta(private$ptr,self$covariance$parameters)
+                           Model__update_u(private$ptr,matrix(rnorm(Model__Q(private$ptr)),ncol=1)) # initialise random effects to random
                            Model__mcmc_set_lambda(private$ptr,self$mcmc_options$lambda)
                            Model__mcmc_set_max_steps(private$ptr,self$mcmc_options$maxsteps)
                            Model__mcmc_set_refresh(private$ptr,self$mcmc_options$refresh)
@@ -1310,6 +1391,7 @@ Model <- R6::R6Class("Model",
                          }
                        },
                        verify_data = function(y){
+                         if(any(is.na(y)))stop("NAs in y")
                          if(self$family[[1]]=="binomial"){
                            if(!(all(y>=0 & y%%1 == 0)))stop("y must be integer >= 0")
                            if(any(y > self$trials))stop("Number of successes > number of trials")
