@@ -1,5 +1,4 @@
-#ifndef GLMMRSPARSE_H
-#define GLMMRSPARSE_H
+#pragma once
 
 #include "general.h"
 // extends some of the SparseChol functions and operators to Eigen classes
@@ -109,4 +108,3 @@ inline sparse submat_sparse(const sparse& A, intvec rows){
 
 }
 
-#endif
