@@ -5,13 +5,12 @@ test_that("model dense ml", {
   df$t1 <- I(df$t==1)*1
   df$t2 <- I(df$t==2)*1
   df$t3 <- I(df$t==3)*1
-  mbptr <- ModelBits__new(
+  mptr <- Model__new_w_pars(
     "t2 + t3 + (1|gr(j))",
     as.matrix(df),
     colnames(df),
     "gaussian","identity",c(0,0,0), c(0.05)
   )
-  mptr <- Model__new_from_bits(mbptr)
   Model__set_y(mptr,y)
   expect_error(Model__update_beta(mptr,c(0,0)))
   Model__update_beta(mptr,c(0.1,0.1,0.1))
@@ -36,13 +35,12 @@ test_that("model dense la", {
   df$t1 <- I(df$t==1)*1
   df$t2 <- I(df$t==2)*1
   df$t3 <- I(df$t==3)*1
-  mbptr <- ModelBits__new(
+  mptr <- Model__new_w_pars(
     "t2 + t3 + (1|gr(j))",
     as.matrix(df),
     colnames(df),
     "gaussian","identity",c(0,0,0), c(0.05)
   )
-  mptr <- Model__new_from_bits(mbptr)
   Model__set_y(mptr,y)
   Model__update_beta(mptr,c(0.1,0.1,0.1))
   Model__update_theta(mptr,c(0.0625))
@@ -67,13 +65,12 @@ test_that("model sparse ml", {
   df$t1 <- I(df$t==1)*1
   df$t2 <- I(df$t==2)*1
   df$t3 <- I(df$t==3)*1
-  mbptr <- ModelBits__new(
+  mptr <- Model__new_w_pars(
     "t2 + t3 + (1|gr(j))",
     as.matrix(df),
     colnames(df),
     "gaussian","identity",c(0,0,0), c(0.05)
   )
-  mptr <- Model__new_from_bits(mbptr)
   Model__set_y(mptr,y)
   expect_error(Model__update_beta(mptr,c(0,0)))
   Model__update_beta(mptr,c(0.1,0.1,0.1))
